<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;
use Illuminate\Support\Facades\Validator;
use DB;
use Storage;
use Yajra\DataTables\DataTables;

class PermissionsController extends Controller
{

    function __construct()
    {
        $this->middleware('auth');
        $this->middleware('permission:permission-list|permission-create|permission-edit|permission-delete', ['only' => ['index','store']]);
        $this->middleware('permission:permission-create', ['only' => ['create','store']]);
        $this->middleware('permission:permission-edit', ['only' => ['edit','update']]);
        $this->middleware('permission:permission-delete', ['only' => ['destroy']]);
        $this->middleware('permission:permission-show', ['only' => ['show']]);
        $this->middleware('permission:permission-ajax-data', ['only' => ['ajaxPermissionData']]);
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $pagetitle   = "Permissions";
        $breadcrumbs = ["Dashboard", "Permissions"];
        $urls        = ['/home', 'permissions-management/permission-list'];
        return view('permission.index', compact('pagetitle', 'breadcrumbs', 'urls'));
    }

    public function ajaxPermissionData()
    {
        $permissions = Permission::all();
        return DataTables::of($permissions)
            ->addColumn('actions', function ($permissions)
            {
                $actions = "<div class='btn-toolbar'>";
                if(auth()->user()->hasPermissionTo('permission-show'))
                {
                    $actions .= '<a class="btn btn-outline-success btn-icon-only" href="' . url("/permissions-management/permission-show", $permissions->id) . '" data-toggle="tooltip" title="View"><i class="fa fa-eye"></i></a>';
                }
                if(auth()->user()->hasPermissionTo('permission-edit'))
                {
                    $actions .= '<a class="btn btn-outline-primary btn-icon-only" href="' . url("/permissions-management/permission-edit", $permissions->id) . '" data-toggle="tooltip" title="Edit"><i class="fa fa-edit"></i></a>';
                }
                if(auth()->user()->hasPermissionTo('permission-delete'))
                {
                    $actions .= '<form method="post" action="' . url('/permissions-management/permission-delete', $permissions->id) . '" class="listing-form">
                                                ' . csrf_field() . '
                                                ' . method_field('delete') . '
                                                <button type="submit" class="btn btn-outline-danger btn-icon-only" data-toggle="tooltip" title="Delete"><i class="fa fa-trash"></i></button>
                                            </form>';
                    $actions .= "</div>";
                }
                return $actions;
            })
            ->addColumn('created_at', function ($permissions) {

                return date('d-m-Y H:i:s', strtotime($permissions->created_at));
            })
            ->rawColumns(['actions'])
            ->make(TRUE);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $permissions = Permission::where('parent_id', 0)->orderBy('parent_id')->get();
        $pagetitle = "Create Permission";
        $breadcrumbs = ["Dashboard","Permissions", 'Create Permission'];
        $urls        = ['/home', 'permissions-management/permission-list', 'permissions-management/permission-create' ];
        return view('permission.create',compact('permissions', 'pagetitle', 'breadcrumbs', 'urls'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $rules = [
            'name' => 'required|max:255|unique:permissions,name',
            'module' => 'required'
        ];

        $validator = $this->validate($request, $rules);

        $permission = new Permission;
        $permission->parent_id = $request->module;
        $permission->name      = $request->name;
        if($permission->save())
        {    
            return redirect('permissions-management/permission-list')->with('alert-success', 'Permission Created Successfully');
        }
        else
        {
            return redirect('permissions-management/permission-create')->with('alert-error', $validator->errors());
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $permission  = Permission::find($id);
        if($permission->parent_id > 0)
        {
            $parent      = Permission::where('id', $permission->parent_id)->first();
        }
        else
        {
            $parent = '';
        }
        $pagetitle   = "View Permission Details";
        $breadcrumbs = ["Dashboard","Permissions", 'View Permission Details'];
        $urls        = ['/home', 'permissions-management/permission-list', 'permissions-management/permission-show/'.$id ];
        return view('permission.show', compact('permission', 'parent', 'pagetitle', 'breadcrumbs', 'urls'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $permissions = Permission::where('parent_id', 0)->orderBy('parent_id')->get();
        $permission  = Permission::find($id);
        if($permission->parent_id > 0)
        {
            $parent      = Permission::where('id', $permission->parent_id)->first();
        }
        else
        {
            $parent = '';
        }
        $pagetitle   = "Edit Permission";
        $breadcrumbs = ["Dashboard","Permissions", 'Edit Permission'];
        $urls        = ['/home', 'permissions-management/permission-list', 'permissions-management/permission-edit/'.$id ];
        return view('permission.edit',compact('permissions', 'parent', 'permission', 'pagetitle', 'breadcrumbs', 'urls'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $rules = [
            'name' => 'required|max:255|unique:permissions,name'.$id,
            'module' => 'required'
        ];

        $validator = $this->validate($request, $rules);

        $permission = Permission::find($id);
        if($permission)
        {
            $permission->parent_id = $request->module;
            $permission->name      = $request->name;
            if($permission->save())
            {    
                return redirect('permissions-management/permission-list')->with('alert-success', 'Permission Updated Successfully');
            }
            else
            {
                return redirect('permissions-management/permission-edit/'.$id)->with('alert-error', $validator->errors());
            }
        }
        else
        {
            return redirect('permissions-management/permission-list')->with('alert-error', 'Permission can not find');
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $permission = Permission::find($id);
        if($permission->delete())
        {    
            return redirect('permissions-management/permission-list')->with('alert-success', 'Permission Deleted Successfully');
        }
    }
}
