<?php
    
namespace App\Http\Controllers;


use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;
use Illuminate\Support\Facades\Validator;
use DB;
use Storage;
use Yajra\DataTables\DataTables;
    
class RoleController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    function __construct()
    {
        $this->middleware('auth');
        $this->middleware('permission:role-list|role-create|role-edit|role-delete', ['only' => ['index','store']]);
        $this->middleware('permission:role-create', ['only' => ['create','store']]);
        $this->middleware('permission:role-edit', ['only' => ['edit','update']]);
        $this->middleware('permission:role-delete', ['only' => ['destroy']]);
        $this->middleware('permission:role-show', ['only' => ['show']]);
        $this->middleware('permission:role-ajax-data', ['only' => ['ajaxRoleData']]);
    }
    
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $pagetitle   = "Roles";
        $breadcrumbs = ["Dashboard", "Roles"];
        $urls        = ['/home', 'roles-management/role-list'];
        return view('roles.index', compact('pagetitle', 'breadcrumbs', 'urls'));
    }
    
    public function ajaxRoleData()
    {
        $roles = Role::all();
        return DataTables::of($roles)
            ->addColumn('actions', function ($roles)
            {
                $actions = "<div class='btn-toolbar'>";
                if(auth()->user()->hasPermissionTo('role-show'))
                {
                    $actions .= '<a class="btn btn-outline-success btn-icon-only" href="' . url("/roles-management/role-show", $roles->id) . '" data-toggle="tooltip" title="View"><i class="fa fa-eye"></i></a>';
                }
                if(auth()->user()->hasPermissionTo('role-edit'))
                {
                    $actions .= '<a class="btn btn-outline-primary btn-icon-only" href="' . url("/roles-management/role-edit", $roles->id) . '" data-toggle="tooltip" title="Edit"><i class="fa fa-edit"></i></a>';
                }
                if(auth()->user()->hasPermissionTo('role-delete'))
                {
                    $actions .= '<form method="post" action="' . url('/roles-management/role-delete', $roles->id) . '" class="listing-form">
                                                ' . csrf_field() . '
                                                ' . method_field('delete') . '
                                                <button type="submit" class="btn btn-outline-danger btn-icon-only" data-toggle="tooltip" title="Delete"><i class="fa fa-trash"></i></button>
                                            </form>';
                    $actions .= "</div>";
                }
                return $actions;
            })
            ->addColumn('created_at', function ($roles) {

                return date('d-m-Y H:i:s', strtotime($roles->created_at));
            })
            ->rawColumns(['actions'])
            ->make(TRUE);
    }
    
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $permissions = Permission::get();
        $pagetitle = "Create Role";
        $breadcrumbs = ["Dashboard","Roles", 'Create Role'];
        $urls        = ['/home', 'roles-management/role-list', 'roles-management/role-create' ];
        return view('roles.create',compact('permissions', 'pagetitle', 'breadcrumbs', 'urls'));
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $rules = [
            'role' => 'required|unique:roles,name',
            'permission' => 'required',
        ];

        $validator = $this->validate($request, $rules);

        $role = new Role;
        $role->name = $request->role;
        if($role->save())
        {    
            $role->syncPermissions($request->permission);
            return redirect('roles-management/role-list')->with('alert-success', 'Role Created Successfully');
        }
        else
        {
            return redirect('roles-management/role-create')->with('alert-error', $validator->errors());
        }
    }
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $role = Role::find($id);
        $rolePermissions = Permission::join("role_has_permissions","role_has_permissions.permission_id","=","permissions.id")
            ->where("role_has_permissions.role_id",$id)
            ->get();
        $pagetitle = "View Role Details";
        $breadcrumbs = ["Dashboard","Roles", 'View Role Details'];
        $urls        = ['/home', 'roles-management/role-list', 'roles-management/role-show/'.$id ];
        return view('roles.show', compact('role', 'rolePermissions', 'pagetitle', 'breadcrumbs', 'urls'));
    }
    
    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $role = Role::find($id);
        $permissions = Permission::get();
        $rolePermissions = DB::table("role_has_permissions")->where("role_has_permissions.role_id",$id)
            ->pluck('role_has_permissions.permission_id','role_has_permissions.permission_id')
            ->all();
        $pagetitle = "Edit Role";
        $breadcrumbs = ["Dashboard","Roles", 'Edit Role'];
        $urls        = ['/home', 'roles-management/role-list', 'roles-management/role-edit/'.$id ];
        return view('roles.edit',compact('role','permissions','rolePermissions', 'pagetitle', 'breadcrumbs', 'urls'));
    }
    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $rules = [
            'role' => 'required',
            'permission' => 'required',
        ];
    
        $validator = $this->validate($request, $rules);
        $role = Role::find($id);
        if($role)
        {
            $role->name = $request->role;
            if($role->save())
            {    
                $role->syncPermissions($request->permission);
                return redirect('roles-management/role-list')->with('alert-success', 'Role Updated Successfully');
            }
            else
            {
                return redirect('roles-management/role-list')->with('alert-error', $validation->errors());
            }
        }
        else
        {
            return redirect('roles-management/role-list')->with('alert-error', 'Role can not find');
        }
    
    }
    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $role = Role::find($id);
        if($role->delete())
        {    
            return redirect('roles-management/role-list')->with('alert-success', 'Role Deleted Successfully');
        }
    }
}