<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use DB;
use Storage;
use Auth;
use App\SocialSetting;
use App\Icon;
use Yajra\DataTables\DataTables;

class SocialSettingsController extends Controller
{
    function __construct()
    {
        $this->middleware('auth');
        $this->middleware('permission:social-setting-list|social-setting-create|social-setting-edit|social-setting-delete', ['only' => ['index','store']]);
        $this->middleware('permission:social-setting-create', ['only' => ['create','store']]);
        $this->middleware('permission:social-setting-edit', ['only' => ['edit','update']]);
        $this->middleware('permission:social-setting-delete', ['only' => ['destroy']]);
        $this->middleware('permission:social-setting-show', ['only' => ['show']]);
        $this->middleware('permission:social-setting-ajax-data', ['only' => ['ajaxSocialData']]);
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $pagetitle   = "Social Setting";
        $breadcrumbs = ["Dashboard", "Social Setting"];
        $urls        = ['/home', 'social-settings/social-setting-list'];
        return view('socialSettings.index', compact('pagetitle', 'breadcrumbs', 'urls'));
    }


    public function ajaxSocialData()
    {
        $socials = SocialSetting::all();
        return DataTables::of($socials)
            ->addColumn('actions', function ($socials)
            {
                $actions = "<div class='btn-toolbar'>";
                if(auth()->user()->hasPermissionTo('role-show'))
                {
                    $actions .= '<a class="btn btn-outline-success btn-icon-only" href="' . url("/social-settings/social-setting-show", $socials->id) . '" data-toggle="tooltip" title="View"><i class="fa fa-eye"></i></a>';
                }
                if(auth()->user()->hasPermissionTo('role-edit'))
                {
                    $actions .= '<a class="btn btn-outline-primary btn-icon-only" href="' . url("/social-settings/social-setting-edit", $socials->id) . '" data-toggle="tooltip" title="Edit"><i class="fa fa-edit"></i></a>';
                }
                if(auth()->user()->hasPermissionTo('role-delete'))
                {
                    $actions .= '<form method="post" action="' . url('/social-settings/social-setting-delete', $socials->id) . '" class="listing-form">
                                                ' . csrf_field() . '
                                                ' . method_field('delete') . '
                                                <button type="submit" class="btn btn-outline-danger btn-icon-only" data-toggle="tooltip" title="Delete"><i class="fa fa-trash"></i></button>
                                            </form>';
                    $actions .= "</div>";
                }
                return $actions;
            })
            ->addColumn('created_at', function ($socials) {

                return date('d-m-Y H:i:s', strtotime($socials->created_at));
            })
            ->rawColumns(['actions'])
            ->make(TRUE);
    }
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $icons = Icon::all();
        $pagetitle = "Create Social Setting";
        $breadcrumbs = ["Dashboard","Social Setting", 'Create Social Setting'];
        $urls        = ['/home', 'social-settings/social-setting-list', 'social-settings/social-setting-create' ];
        return view('socialSettings.create',compact('icons', 'pagetitle', 'breadcrumbs', 'urls'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $rules = [
            'title' => 'required|unique:social_settings,title',
            'description' => 'required',
            'link' => 'required',
            'icon' => 'required'
        ];

        $validator = $this->validate($request, $rules);

        $socialSetting = new SocialSetting;
        $socialSetting->title = $request->title;
        $socialSetting->description = $request->description;
        $socialSetting->link = $request->link;
        $socialSetting->icon = $request->icon;
        if($socialSetting->save())
        {    
            return redirect('social-settings')->with('alert-success', 'Social Settings Created Successfully');
        }
        else
        {
            return redirect('social-settings/social-setting-create')->with('alert-error', $validator->errors());
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $social = SocialSetting::find($id);
        $pagetitle = "View Social Setting Details";
        $breadcrumbs = ["Dashboard","Social Setting", 'View Social Setting Details'];
        $urls        = ['/home', 'social-settings/social-setting-list', 'social-settings/social-setting-show/'.$id ];
        return view('socialSettings.show', compact('social', 'pagetitle', 'breadcrumbs', 'urls'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $social = SocialSetting::find($id);
        $icons = Icon::all();
        $pagetitle = "Edit Social Setting";
        $breadcrumbs = ["Dashboard","Social Setting", 'Edit Social Setting'];
        $urls        = ['/home', 'social-settings/social-setting-list', 'social-settings/social-setting-edit/'.$id ];
        return view('socialSettings.edit',compact('social','icons', 'pagetitle', 'breadcrumbs', 'urls'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $rules = [
            'title' => 'required|unique:social_settings,title,'.$id,
            'description' => 'required',
            'link' => 'required',
            'icon' => 'required'
        ];

        $validator = $this->validate($request, $rules);

        $socialSetting = SocialSetting::find($id);
        if($socialSetting)
        {
            $socialSetting->title = $request->title;
            $socialSetting->description = $request->description;
            $socialSetting->link = $request->link;
            $socialSetting->icon = $request->icon;
            if($socialSetting->update())
            {    
                return redirect('social-settings')->with('alert-success', 'Social Settings Updated Successfully');
            }
            else
            {
                return redirect('social-settings/social-setting-create')->with('alert-error', $validator->errors());
            }
        }
        else
        {
            return redirect('social-settings/social-setting-list')->with('alert-error', 'Social Setting can not find');
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $SocialSetting = SocialSetting::find($id);
        if($SocialSetting->delete())
        {    
            return redirect('social-settings/social-setting-list')->with('alert-success', 'Social Setting Deleted Successfully');
        }
    }
}
