<?php
    
namespace App\Http\Controllers;
    
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Spatie\Permission\Models\Role;
use Illuminate\Support\Facades\Validator;
use Yajra\DataTables\DataTables;
use Illuminate\Support\Arr;
use App\User;
use App\Address;
use DB;
use Hash;
use Storage;
use Auth;
use Mail;
    
class UserController extends Controller
{
    function __construct()
    {
        $this->middleware('auth');
        $this->middleware('permission:user-list|user-create|user-edit|user-delete|user-profile|user-profile-update', ['only' => ['index','store']]);
        $this->middleware('permission:user-create', ['only' => ['create','store']]);
        $this->middleware('permission:user-edit', ['only' => ['edit','update']]);
        $this->middleware('permission:user-status', ['only' => ['status','status']]);
        $this->middleware('permission:user-profile', ['only' => ['profile','profile']]);
        $this->middleware('permission:user-profile-update', ['only' => ['profile','profile_update']]);
        $this->middleware('permission:user-delete', ['only' => ['destroy']]);
        $this->middleware('permission:user-show', ['only' => ['show']]);
        $this->middleware('permission:user-ajax-data', ['only' => ['ajaxUserData']]);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $pagetitle   = "Users";
        $breadcrumbs = ["Dashboard", "Users"];
        $urls        = ['/home', 'users-management/user-list'];
        return view('users.index', compact('pagetitle', 'breadcrumbs', 'urls'));
    }

    public function ajaxUserData()
    {
        $users = User::all();
        return DataTables::of($users)
            ->addColumn('status', function ($users) {
                return $users->status  == 1 ? "Active" : "Disable";
            })
            ->addColumn('actions', function ($users)
            {
                $actions = "<div class='btn-toolbar'>";
                if(auth()->user()->hasPermissionTo('user-status'))
                {
                    $actions .= '<label class="custom-toggle '.( $users->status == 1 ? 'custom-toggle-success' : '') .'"><input name="changeStatus" id="changeStatus" data-id="'.$users->id.'" type="checkbox" onClick="change_status(this);" '.( $users->status == 1 ? 'checked=""' : '') .'>
                    <span class="custom-toggle-slider rounded-circle" data-label-off="No" data-label-on="Yes"></span>
                    </label>';
                }
                if(auth()->user()->hasPermissionTo('user-show'))
                {
                    $actions .= '<a class="btn btn-outline-success btn-icon-only" href="' . url("/users-management/user-show", $users->id) . '" data-toggle="tooltip" title="View"><i class="fa fa-eye"></i></a>';
                }
                if(auth()->user()->hasPermissionTo('user-edit'))
                {
                    $actions .= '<a class="btn btn-outline-primary btn-icon-only" href="' . url("/users-management/user-edit", $users->id) . '" data-toggle="tooltip" title="Edit"><i class="fa fa-edit"></i></a>';
                }
                if(auth()->user()->hasPermissionTo('user-delete'))
                {
                    $actions .= '<form method="post" action="' . url('/users-management/user-delete', $users->id) . '" class="listing-form">
                                                ' . csrf_field() . '
                                                ' . method_field('delete') . '
                                                <button type="submit" class="btn btn-outline-danger btn-icon-only" data-toggle="tooltip" title="Delete"><i class="fa fa-trash"></i></button>
                                            </form>';
                    $actions .= "</div>";
                }
                return $actions;
            })
            ->addColumn('created_at', function ($users) {

                return date('d-m-Y H:i:s', strtotime($users->created_at));
            })
            ->rawColumns(['actions'])
            ->make(TRUE);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $roles = Role::all();
        $pagetitle = "Create User";
        $breadcrumbs = ["Dashboard","User", 'Create User'];
        $urls        = ['/home', 'users-management/user-list', 'users-management/user-create' ];
        return view('users.create',compact('roles', 'pagetitle', 'breadcrumbs', 'urls'));
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $rules = [
            'first_name' => 'required|regex:/^[\pL\s\-]+$/u|max:30',
            'last_name' => 'required|regex:/^[\pL\s\-]+$/u|max:30',
            'email' => 'required|email|unique:users,email|max:50',
            'phone' => 'required|numeric|digits_between:1,15',
            'address' => 'required|max:255',
            'zip' => 'required|max:30',
            'unit' => 'required|max:30',
            'company' => 'required|regex:/^[\pL\s\-]+$/u|max:30',
            'country' => 'required|regex:/^[\pL\s\-]+$/u|max:100',
            'state' => 'required|regex:/^[\pL\s\-]+$/u|max:100',
            'city' => 'required|regex:/^[\pL\s\-]+$/u|max:100',
            'roles' => 'required',
            'password' => 'required|min:8|confirmed'
        ];

        $validator = $this->validate($request, $rules);
        $user = User::create([
            'first_name' => $request->first_name,
            'last_name' => $request->last_name,
            'email' => $request->email,
            'phone' => $request->phone,
            'password' => Hash::make($request->password),
        ]);

        $address = new Address;
        $address->user_id = $user->id;
        $address->name    = $request->first_name.' '.$request->last_name;
        $address->phone   = $request->phone;
        $address->company = $request->company;
        $address->street1 = $request->address;
        $address->unit    = $request->unit;
        $address->city    = $request->city;
        $address->state   = $request->state;
        $address->country = $request->country;
        $address->zip     = $request->zip;
        $address->save();

        $user->assignRole($request->roles);
        $user->sendEmailVerificationNotification();

        if($user)
        {    
            return redirect('users-management/user-list')->with('alert-success', 'User Created Successfully');
        }
        else
        {
            return redirect('users-management/user-create')->with('alert-error', $validator->errors());
        }
    }
    
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $user = User::find($id);
        $roles = Role::all();
        $userRole = $user->roles->pluck('name')->all();
        $pagetitle = "View User Details";
        $breadcrumbs = ["Dashboard","Users", 'View User Details'];
        $urls        = ['/home', 'users-management/user-list', 'users-management/user-show/'.$id ];
        return view('users.show', compact('user', 'roles', 'userRole', 'pagetitle', 'breadcrumbs', 'urls'));
    }
    
    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = User::find($id);
        $roles = Role::all();
        $userRole = $user->roles->pluck('name')->all();
        $pagetitle = "Edit User";
        $breadcrumbs = ["Dashboard","Users", 'Edit User'];
        $urls        = ['/home', 'users-management/user-list', 'users-management/user-edit/'.$id ];
        return view('users.edit',compact('user','roles','userRole', 'pagetitle', 'breadcrumbs', 'urls'));
    }
    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $rules = [
            'first_name' => 'required|string|max:30',
            'last_name' => 'required|string|max:30',
            'email' => 'required|email|max:50|unique:users,email,'.$id,
            'phone' => 'required|numeric|digits_between:1,15',
            'address' => 'required|string|max:255',
            'zip' => 'required|string|max:50',
            'unit' => 'required|string|max:50',
            'company' => 'required|string|max:50',
            'country' => 'required|string|max:100',
            'state' => 'required|string|max:100',
            'city' => 'required|string|max:100',
            'roles' => 'required'
        ];

        $validator = $this->validate($request, $rules);
        $user = User::find($id);
        if($user)
        {
            $user->first_name = $request->first_name;
            $user->last_name  = $request->last_name;
            $user->email      = $request->email;
            $user->phone      = $request->phone;
            $user->password   = Hash::make($request->password);
            $user->save();
            
            $address = Address::where('user_id', $user->id)->first();
            if($address)
            {
                $address->name    = $request->first_name.' '.$request->last_name;
                $address->phone   = $request->phone;
                $address->company = $request->company;
                $address->street1 = $request->address;
                $address->unit    = $request->unit;
                $address->city    = $request->city;
                $address->state   = $request->state;
                $address->country = $request->country;
                $address->zip     = $request->zip;
                $address->save();
            }

            DB::table('model_has_roles')->where('model_id',$id)->delete();
            
            $user->assignRole($request->roles);

            if($user)
            {    
                return redirect('users-management/user-list')->with('alert-success', 'User updated Successfully');
            }
            else
            {
                return redirect('users-management/user-edit/'.$id)->with('alert-error', $validator->errors());
            }
        }
        else
        {
            return redirect('users-management/user-list')->with('alert-error', 'User can not find');
        }
    }
    
    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $user = User::find($id);
        $address = Address::where('user_id', $user->id)->first();
        if($address)
        {
            $address->delete();
        }
        if($user->delete())
        {    
            return redirect('users-management/user-list')->with('alert-success', 'User Deleted Successfully');
        }
    }

    public function profile($id)
    {
        $user = User::find($id);
        $pagetitle = "User Profile Edit";
        $breadcrumbs = ["Dashboard","Users", 'User Profile Edit'];
        $urls        = ['/home', 'users-management/user-list', 'users-management/user-profile/'.$id ];
        return view('users.profile',compact('user', 'pagetitle', 'breadcrumbs', 'urls'));
    }

    public function profile_update(Request $request, $id)
    {
        $rules = [
            'first_name' => 'required|string|max:30',
            'last_name' => 'required|string|max:30',
            'email' => 'required|email|max:50|unique:users,email,'.$id,
            'phone' => 'required|numeric|digits_between:1,15',
            'address' => 'required|string|max:255',
            'zip' => 'required|string|max:50',
            'unit' => 'required|string|max:50',
            'company' => 'required|string|max:50',
            'country' => 'required|string|max:100',
            'state' => 'required|string|max:100',
            'city' => 'required|string|max:100',
        ];

        if(isset($request->password))
        {
            $rules['password'] = 'required|min:8|confirmed';
        }

        $validator = $this->validate($request, $rules);
        $user = User::find($id);
        if($user)
        {
            $user->first_name = $request->first_name;
            $user->last_name  = $request->last_name;
            $user->email      = $request->email;
            $user->phone      = $request->phone;
            if(isset($request->password))
            {
                $user->password   = Hash::make($request->password);
            }
            // Update profile image
            if($request->hasFile('image'))
            {
                $image            = $request->file('image');
                $destination_path = 'public/users';
                $filename = imageUpload($image,$destination_path);
                $user->avatar = "users/".$filename;
            }
            $user->update();
            
            $address = Address::where('user_id', $user->id)->first();
            if($address)
            {
                $address->name    = $request->first_name.' '.$request->last_name;
                $address->phone   = $request->phone;
                $address->company = $request->company;
                $address->street1 = $request->address;
                $address->unit    = $request->unit;
                $address->city    = $request->city;
                $address->state   = $request->state;
                $address->country = $request->country;
                $address->zip     = $request->zip;
                $address->update();
            }

            if($user)
            {    
                return redirect('users-management/user-profile/'.$id)->with('alert-success', 'User Profile updated Successfully');
            }
            else
            {
                return redirect('users-management/user-profile/'.$id)->with('alert-error', $validator->errors());
            }
        }
        else
        {
            return redirect('users-management/user-list')->with('alert-error', 'User can not find');
        }
    }

    public function status($id)
    {
        $user = User::find($id);

        $data['slot']= "Hello ".$user->first_name." ".$user->last_name;
        $data['email']= $user->email;

        if($user->status == 1)
        {
            $data['subcopy']= "You are receiving this email because we received a account deactivation request for your account. Now your account is deactivated";
            Mail::send('vendor.mail.html.default', $data, function($message) use($data) {
                $message->from(env('MAIL_FROM_ADDRESS'));
                $message->to($data['email']);
                $message->subject('Account Deactivation Notification');
            });

            $user->status = 0;
        }
        else
        {
            $data['subcopy']= "You are receiving this email because we received a account activation request for your account. Now your account is activated";
            Mail::send('vendor.mail.html.default', $data, function($message) use($data) {
                $message->from(env('MAIL_FROM_ADDRESS'));
                $message->to($data['email']);
                $message->subject('Account Activation Notification');
            });

            $user->status = 1;
        }
        $user->update();
        return array('is_active' => $user->status);
    }
}